<?php

/*
 * iPay88 Gateway for WooCommerce
 *
 * Description: Allows you to use iPay88 Over The Counter Payment with the WooCommerce plugin.
 * Plugin URI: https://www.ipay88.co.id/
 * Author: iPay88 Indonesia
 * License: The MIT License (MIT)
 *
 */
 
class ipay88_over_the_counter_Gateway extends ipay88_over_the_counter_Abstract 
{
	//editor: Alda Mustafri
	
    public $id 			= 'ipay88_over_the_counter';
    public $has_fields 	= true;
	
    protected $ipay88_payment_method_selected;

    public function __construct() 
	{
        parent::__construct();

        $this->icon 				= IPAY88_OVER_THE_COUNTER_PLUGIN_URL . '/assets/images/over_the_counter-menu.png';
        $this->method_title 		= __('iPay88 Over The Counter Payment', IPAY88_OVER_THE_COUNTER_TEXT_DOMAIN);
        $this->method_description 	= __('Allows you to use iPay88 Over The Counter Payment with the WooCommerce plugin.', IPAY88_OVER_THE_COUNTER_TEXT_DOMAIN);
       
	    $this->title = isset($this->settings['title']) ? $this->settings['title'] : "iPay88 Over The Counter Payment.";
        $this->notify_url = add_query_arg('wc-api', 'ipay88_over_the_counter_Gateway', home_url('/'));
		
        // Our Actions
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
        add_action('woocommerce_api_' . $this->id . '_gateway', array($this, 'ipn_response'));
    	
		$this->hash_amount 		= 0;
		$this->formatted_amount = 0;
	}

    //Initial ipay88 Configs
    function init_configs($isPingback = false) 
	{
        ipay88_over_the_counter_Config::getInstance()->set(array( 'merchant_code' => $this->settings['merchant_code'],
            									 'merchant_key' => $this->settings['merchant_key']));
    }

    function getConfigOptionByKey($key) 
	{
        return $this->settings[$key];
    }

    //@param $order_id
    function receipt_page($order_id) 
	{
        $this->init_configs();
        echo $redirect_form = $this->preparePayment($order_id);
    }

    function dd($data) 
	{
        echo "<pre>";
        print_r($data);
        echo "</pre>";
    }

    function preparePayment($order_id) 
	{
        $merchantCode 	= $this->getConfigOptionByKey('merchant_code');
        $merchantKey 	= $this->getConfigOptionByKey('merchant_key');
        $refNo 			= $order_id;
		$paymentId 		= $_GET['ipay88_payment_method_selected'];
        $order 			= wc_get_order($order_id);
		$isTest 		= $this->getConfigOptionByKey('test_mode');
		
		if($isTest == 'no')
		{
			$getwayUrl = ipay88_over_the_counter_Indonesia::URL_PAYMENT_PRODUCTION;
		}
		
		else
		{
			$getwayUrl = ipay88_over_the_counter_Indonesia::URL_PAYMENT_SANDBOX;
		}
		       
	    //$amount 		= $order->get_total();
        //$hashAmount 	= str_replace(".", "", str_replace(",", "", $amount));
       
	   	// Format the order total
		$this->format_amount($order->get_total());
		$hashAmount		= $this->formatted_amount;
	   	   
	    $currency 		= $order->get_data()['currency'];
        
		//generate signature
		$str 				= $merchantKey.$merchantCode.$refNo.$hashAmount.$currency;
		$signature 			= new ipay88_over_the_counter_Signature($str);
        $requestSignature 	= $signature->getSignature();

        $userInfo 		= $this->get_billing_address($order);
        $backendUrl 	= $this->notify_url;
        $responseUrl  	= $this->notify_url . '&responseURL=true';

        $data = array(
						'MerchantCode'  => $merchantCode,
						'PaymentId'     => $paymentId,
						'RefNo'         => $refNo,
						'Amount'        => $hashAmount,
						'Currency'      => $currency,
						'ProdDesc'      => 'Order #'. $order_id,
						'UserName'      => $userInfo['billing_first_name'] . ' ' . $userInfo['billing_last_name'],
						'UserEmail'     => $userInfo['billing_email'],
						'UserContact'   => $userInfo['billing_phone'],
						'Lang'          => 'UTF-8',
						'Signature'     => $requestSignature,
						'ResponseURL'   => $responseUrl,
						'BackendURL'    => $backendUrl,
					);

        $wiget = new ipay88_over_the_counter_Wiget($data);
        $wiget->setAction($getwayUrl);

        return $wiget->generateRedirectForm();
    }

    // Process the order after payment is made
    // @param int $order_id
    // @return array
    function process_payment($order_id) 
	{
        $order = wc_get_order($order_id);
		
		$payment_method_selected 			= $_POST['ipay88_bank_payment_method'];
        $_SESSION['posted_payment_type']	= $_POST['ipay88_bank_payment_method'];

        return array(	'result' 	=> 'success',
            			'redirect' 	=> add_query_arg('key', $order->order_key, add_query_arg(array(
                        'order-pay' => !method_exists($order, 'get_id') ? $order->id : $order->get_id(),
                        'ipay88_payment_method_selected' => $payment_method_selected),
                    	$order->get_checkout_payment_url(true))));
    }

    // Check the response from ipay88's Servers
    function ipn_response() 
	{
        global $woocommerce;

        $order = wc_get_order($refNo = $_POST['RefNo']);

        if (!$order) 
		{
            die('The order is Invalid!');
        }

        $merchantKey = $this->getConfigOptionByKey('merchant_key');
        $transaction = new ipay88_over_the_counter_Transaction($_POST);

        $transaction->setMerchantKey($merchantKey);
        $transaction->setSignature($_POST['Signature']);

        if($transaction->isPaid()) 
		{
            $order->add_order_note(__('The Order has been Paided by ipay88 Payment Gateway - Transaction Id: ' . $_POST['TransId'], IPAY88_OVER_THE_COUNTER_TEXT_DOMAIN));

            $order->payment_complete();
            $woocommerce->cart->empty_cart();

            if(isset($_REQUEST['responseURL']) && $_REQUEST['responseURL'] == 'true') 
			{
                status_header(200);
                wp_redirect( $this->get_return_url( $order ) );

                // Return thank you page redirect
                return array( 'result'   => 'success', 'redirect' => $this->get_return_url($order));
            } 
			
			else 
			{
                die('RECEIVEOK');
            }
        } 
		
		elseif ($transaction->isWaitingPayment()) 
		{
            $order->add_order_note(__('The Order is waiting payment via ipay88 Payment Gateway - Transaction Id: ' . $_POST['TransId'], IPAY88_OVER_THE_COUNTER_TEXT_DOMAIN));
			
            $order->update_status('on-hold');
            wp_redirect( $this->get_return_url( $order ) );
        } 
		
		else 
		{
            $order->cancel_order(__('Reason: ' . $_POST['ErrDesc'], IPAY88_OVER_THE_COUNTER_TEXT_DOMAIN));
            return wp_redirect(WC()->cart->get_checkout_url());
        }
    }
	
	// Format the two amounts we need.
	// One for hashing
	// One for request parameter
	// @param double $amount
	function format_amount($amount) 
	{
		if (is_numeric($amount)) 
		{
			$this->hash_amount 		= number_format($amount, 2, '', '');
			$this->formatted_amount = number_format($amount, 2, '', '');
		}
	}
}
